//-----------------------------------------------------------------------------
// DiskGeom - list disk geometry information
// Author:  DLang   2009   
//-----------------------------------------------------------------------------

#include "atlstr.h"
#include "winioctl.h"

#include <iostream>
using namespace std;

//-----------------------------------------------------------------------------
DISK_GEOMETRY_EX* GetDiskGeometry(int nDriveNo, DWORD& geomSize)
{
	DWORD	dwWritten;
	BOOL	ret;
	HANDLE	hDevice;
	TCHAR	pszDevice[256];

    // Struture is variable size, so guess at max size.
    geomSize = sizeof(DISK_GEOMETRY_EX) + 4069;
    DISK_GEOMETRY_EX* psDiskGeometryEx = (DISK_GEOMETRY_EX*)malloc(geomSize);

	if (psDiskGeometryEx == NULL)
		return	NULL;
	::ZeroMemory(psDiskGeometryEx, geomSize);

	::_stprintf_s(pszDevice, 256, _T("\\\\.\\PhysicalDrive%d"), nDriveNo);
	hDevice = ::CreateFile(pszDevice, 
            0,      
            FILE_SHARE_READ | FILE_SHARE_WRITE, 
            NULL, 
            OPEN_EXISTING, 
            0, 
            NULL);
	
    if (hDevice == INVALID_HANDLE_VALUE)
    {
        free(psDiskGeometryEx);
		return	NULL;
    }

	ret = ::DeviceIoControl(
        hDevice,
        IOCTL_DISK_GET_DRIVE_GEOMETRY_EX,
        NULL,
        0,
        psDiskGeometryEx,
        geomSize,
        &dwWritten,
        NULL);

	if (ret == FALSE || dwWritten < sizeof(DISK_GEOMETRY_EX))
	{
		ret = ::DeviceIoControl(
            hDevice,
            IOCTL_DISK_GET_DRIVE_GEOMETRY,
            NULL,
            0,
            &psDiskGeometryEx->Geometry,
            sizeof(DISK_GEOMETRY),
            &dwWritten,
            NULL);

		if (ret == FALSE || dwWritten != sizeof(DISK_GEOMETRY))
		{
			::ZeroMemory(psDiskGeometryEx,sizeof(DISK_GEOMETRY_EX));
			ret = FALSE;
		}
		else
		{
			psDiskGeometryEx->DiskSize.QuadPart = psDiskGeometryEx->Geometry.Cylinders.QuadPart;
			psDiskGeometryEx->DiskSize.QuadPart *= psDiskGeometryEx->Geometry.TracksPerCylinder;
			psDiskGeometryEx->DiskSize.QuadPart *= psDiskGeometryEx->Geometry.SectorsPerTrack;
			psDiskGeometryEx->DiskSize.QuadPart *= psDiskGeometryEx->Geometry.BytesPerSector;
		}
	}

	::CloseHandle(hDevice);

	return	psDiskGeometryEx;
}

//-----------------------------------------------------------------------------
bool DisplayDiskGeometry(int driveN)
{
    DISK_GEOMETRY_EX* pDiskGeom = NULL;
    DWORD size;

    pDiskGeom = GetDiskGeometry(driveN, size);
    if (pDiskGeom != NULL)
    {
        PDISK_PARTITION_INFO pDiskPart = DiskGeometryGetPartition(pDiskGeom);
        PDISK_DETECTION_INFO pDiskInfo = DiskGeometryGetDetect(pDiskGeom);

        std::cout 
            << "\nCylinders:   \t" 
            << pDiskGeom->Geometry.Cylinders.QuadPart
            << "\nMediaType:   \t" 
            << pDiskGeom->Geometry.MediaType
            << "\nTracks/Cyl:  \t" 
            << pDiskGeom->Geometry.TracksPerCylinder
            << "\nSectors/Trk: \t" 
            << pDiskGeom->Geometry.SectorsPerTrack
            << "\nBytes/Sector:\t" 
            << pDiskGeom->Geometry.BytesPerSector
            << "\n";
   
        free(pDiskGeom);
        return true;
    }
    else
    {
        return false;
    }
}
